package evolvekernel;

/**

 The code to read in the training and testing data. It runs SVM (LibSVM)
 with parameters and GP Inidviduals as kernels using a Threaded evaluator.
*/

import ec.*;
import ec.util.*;
import ec.gp.*;
import ec.gp.koza.*;
import ec.simple.*;
import libsvm.*;
import sim.util.*;
import java.util.*;
import java.io.*;

public class SVMEvaluator extends GPProblem implements SimpleProblemForm {

	static svm_parameter[] params;
	static svm_problem[] prob;
	static svm_model[] model;

	public static final String P_FILENAME = "train-filename";
	public static final String P_TEST_FILENAME = "test-filename";
	public static final String P_C_VALUE = "cvalue";
	public static final String P_NR_FOLD = "nr-fold";
	public static final String GAMMA = "gamma";
	public static final String NU = "NU";

	transient static Bag testData;
	static int nr_fold;
	public transient ThreadEvaluator[] threadedEval;
	boolean[] startedThread;

	public static svm_node[] x;
	public static svm_node[] y;
	
	Vector vy = new Vector();
	Vector vx = new Vector();

	public static svm_problem getProblem(int thread) {
		return prob[thread];
	}

	public static svm_parameter getParams(int thread) {
		return params[thread];
	}

	public static Bag getTestData() {
		return testData;
	}

	public static final long TOO_LONG = 1000 * 10 * 60;

	public Object protoClone() throws CloneNotSupportedException {
		SVMEvaluator svm = (SVMEvaluator) super.clone();
		return svm;
	}
	
	private void initializeProblem(int k){
		prob[k] = new svm_problem();
		prob[k].l = vy.size();
		prob[k].x = new svm_node[prob[k].l][];

		for (int i = 0; i < prob[k].l; i++)
			prob[k].x[i] = (svm_node[]) vx.elementAt(i);
		prob[k].y = new double[prob[k].l];
		for (int i = 0; i < prob[k].l; i++) {
			prob[k].y[i] = ((Double)vy.elementAt(i)).doubleValue();
		}
	}
	
	private void initializeParam(int i, final EvolutionState state, final Parameter base){
		params[i] = new svm_parameter();
		params[i].svm_type = svm_parameter.C_SVC;
		params[i].kernel_type = svm_parameter.POLY;
		params[i].degree = 3;
		params[i].gamma = 0; // 1/k
		params[i].coef0 = 0;
		if (base != null)
			params[i].nu = state.parameters.getDouble(base.push(NU), null, 0);
		else
			params[i].nu = 0.5;
		params[i].cache_size = 100;
		if (base != null)
			params[i].C = state.parameters
					.getDouble(base.push(P_C_VALUE), null, 0);
		else
			params[i].C = 1.0;
		if (base != null)
			params[i].gamma = state.parameters
					.getDouble(base.push(GAMMA), null, 0);
		else
			params[i].gamma = 0.005;
		params[i].eps = 1e-3;
		params[i].p = 0.1;
		params[i].shrinking = 1;
		params[i].probability = 0;
		params[i].nr_weight = 0;
		params[i].weight_label = new int[0];
		params[i].weight = new double[0];
	}

	public void setup(final EvolutionState state, final Parameter base) {
		if (base != null)
			super.setup(state, base);
		if (base != null)
			nr_fold = state.parameters.getInt(base.push(P_NR_FOLD), null);
		else
			nr_fold = 3;
		String trainFilename = "";
		if (base != null)
			trainFilename = state.parameters.getString(base.push(P_FILENAME),
					null);
		else
			trainFilename = "MotifWithHS200Scaled.libsvm";
		int numberOfThreads = state.parameters.getInt(base.push("threads"), null);
		params = new svm_parameter[numberOfThreads];
		for(int i=0; i< numberOfThreads; i++){
			initializeParam(i,state,base);
		}
		

		int maxIndex = 1;
		int minIndex = 0;
		double maxValue = 1;
		double minValue = 0;

		// read the training data and setup the model and problem
		try {
			BufferedReader fp = new BufferedReader(
					new FileReader(trainFilename));
		
			int max_index = 0;


			while (true) {
				String line = fp.readLine();
				if (line == null)
					break;

				max = 0;
				int index;
				int max = 0;
				String col;
				double value;
				double[] result;
				StringTokenizer tok = new StringTokenizer(line, " \t");
				tok.nextToken(); // skip class
				while (tok.hasMoreTokens()) {
					col = tok.nextToken();
					index = Integer
							.parseInt(col.substring(0, col.indexOf(":")));
					if (index > max)
						max = index;
				}


				// read values into array
				tok = new StringTokenizer(line, " \t");

				// 1. class
				double classValue = Double.parseDouble(tok.nextToken());
				vy.add(classValue);
				svm_node[] x = new svm_node[500];

				// 2. attributes
				int j = 0;
				x = initAllNodes(x);
				while (tok.hasMoreTokens()) {
					col = tok.nextToken();
					index = Integer
							.parseInt(col.substring(0, col.indexOf(":")));
					value = Double.parseDouble(col
							.substring(col.indexOf(":") + 1));
					// x[j] = new svm_node();
					x[j].index = index;
					x[j].value = value;
					j = j + 1;
				}
				if (x.length < 499)
					System.out.println("Soemthing wrong!");
				vx.addElement(x);
			}			
			this.prob =new svm_problem[numberOfThreads];
			for(int k=0; k< numberOfThreads; k++){
				initializeProblem(k);
			}
			
			fp.close();
		} catch (IOException e) {
			state.output.fatal("Error in setting up training data: " + e);
		}
		for(int k=0; k< numberOfThreads; k++){
			// check parameters and the model
			String msg = svm.svm_check_parameter(prob[k], params[k]);
			if (msg != null)
				state.output.fatal("Error with parameters: " + msg);
		}
		

		// read in test data
		minIndex = 0;
		maxIndex = 1;
		minValue = 0;
		maxValue = 1;
		startedThread = new boolean[numberOfThreads];
		this.threadedEval = new ThreadEvaluator[numberOfThreads];
		for(int i=0; i< numberOfThreads; i++){
			this.getThreadedEval(i).start();
			startedThread[i] = true;
		}
		
		
		
	}

	static double max = -1;

	private svm_node[] initAllNodes(svm_node[] nodes) {
		for (int i = 0; i < nodes.length; i++) {
			nodes[i] = new svm_node();
			nodes[i].index = i + 1;
			nodes[i].value = 0;
		}
		return nodes;

	}

	private ThreadEvaluator getThreadedEval(int i) {
		if (this.threadedEval[i] == null) {
			this.threadedEval[i] = new ThreadEvaluator("svm" +i);
		}
		return this.threadedEval[i];
	}

	/**
	 * This method creates a thread to evaluate the SVM cross validation
	 * It waits for 15 minutes, and if no results, kills the thread
	 * and assigns a low fitness. This can be a boon or bane based on the 
	 * problem. It makes sense to run this problem and see how
	 * each kernel and complex combinations work before changing this parameter
	 */
	public void evaluate(final EvolutionState state, final Individual ind,
			final int subpopulation, final int threadnum) {
		if (ind.evaluated)
			return;
		
		if(this.threadedEval == null){
			//can happen only on serialization
			this.setup(state, null);
		} 

		if ( !this.startedThread[threadnum]) {
			initializeParam(threadnum,state,null);
			initializeProblem(threadnum);
			getThreadedEval(threadnum).start();
			this.startedThread[threadnum] = true;
		}

	
		double accuracy = 0;
		double correctCount = 0;
		long currenttime = 0;
		long starttime = 0;
		GPIndividual gpInd = (GPIndividual)ind;
		if (nr_fold > 0) {
			double[] target = new double[prob[threadnum].l];
			// evaluate the child one as it give the C value
			KernelData cTreeData = new KernelData();			
			//evaluate the C parameter
			gpInd.trees[1].child.eval(state, threadnum,
					cTreeData,this.stack,gpInd,this );
			//set it to the right params
			params[threadnum].C = cTreeData.val;
			KernelData kernelTreeData = new KernelData();			
			this.threadedEval[threadnum].setParameter(params[threadnum]);
			this.threadedEval[threadnum].setProblem(prob[threadnum]);
			this.threadedEval[threadnum].setTarget(target);
			this.threadedEval[threadnum].setData(kernelTreeData);
			this.threadedEval[threadnum].stack = this.stack;
			this.threadedEval[threadnum].state = state;
			this.threadedEval[threadnum].individual= gpInd;
			this.threadedEval[threadnum].gpProblem = this;
			this.threadedEval[threadnum].setNumberOfFolds(nr_fold);
			this.threadedEval[threadnum].setReady();
			starttime = System.currentTimeMillis();
			//go in a loop to to evaluate
			while (true) {
				try {
					Thread.sleep(50);
					if (!this.threadedEval[threadnum].getReady()) {
						break;
					}
					currenttime = System.currentTimeMillis();
					// get the start time					
					if ((currenttime - starttime) > (15 * 60 * 1000)) {
						this.threadedEval[threadnum].suspend();
						this.threadedEval[threadnum].stop();
						this.threadedEval[threadnum] = null;
						this.startedThread[threadnum] = false;						
						break;
					}

				} catch (Exception e) {
					e.printStackTrace();
				}

			}
			
		}
	
		// set the fitness
		if (this.startedThread[threadnum]) {
			KozaFitness f = ((KozaFitness) ind.fitness);
			accuracy = this.threadedEval[threadnum].getAccuracy();
			if (accuracy != 0)
				f.setStandardizedFitness(state, (float) (1.0 / accuracy));
			else
				f.setStandardizedFitness(state, (float) 1000000);

			f.hits = prob[threadnum].l;
			ind.evaluated = true;
			KernelPrinter.writeFeature(gpInd, state, (currenttime - starttime), accuracy, 0);
		} else {
			KozaFitness f = ((KozaFitness) ind.fitness);
			accuracy = 0;
			f.setStandardizedFitness(state, (float) 1000000);
			f.hits = prob[threadnum].l;
			KernelPrinter.writeFeature(gpInd, state, (currenttime - starttime), accuracy, 1);
		}
	}
}

class Pair {
	double cls;
	svm_node[] vector;

	Pair(double c, svm_node[] v) {
		cls = c;
		vector = new svm_node[v.length];
		for (int i = 0; i < v.length; i++) {
			vector[i] = new svm_node();
			vector[i].index = v[i].index;
			vector[i].value = v[i].value;
		}
	}
}
