package evolvekernel; 
import ec.EvolutionState;
import ec.Problem;
import ec.gp.ADFStack;
import ec.gp.GPIndividual;
import libsvm.svm;
import libsvm.svm_node;
import libsvm.svm_parameter;
import libsvm.svm_problem;

/**
 * This class does the actual computation of SVM cross validation 
 * training. It invokes svm with GP Inidividuals, and times them.
 * Based on timeout, this can get stopped and removed by the invoker.
 * This has advantage/disadvantage of stopping some huge kernel computation
 * from taking up resource. Also sometimes good kernel may also be stopped
 * So user needs to run it and validate the results and adjust the time limit.
 * @author udaykamath
 *
 */

public class ThreadEvaluator extends Thread{
	public svm_problem problem;
	public double[] target;
	public svm_parameter param;
	public svm svmObject;
	public KernelData data;
	public boolean ready;
	double accuracy =0;
	public long startTime;
	public long endTime;
	public GPIndividual individual;
	public int threadNumber;
	public static svm_node[] x;
	public static svm_node[] y;
	public EvolutionState state;
	public ADFStack stack;
	public static Problem gpProblem;
	private int folds;
	
	public ThreadEvaluator(String evalName){
		super(evalName);
	}
	
	public void run() {
		while(true){
			if(!isReady()){
				try{
					Thread.sleep(20);
				}catch(Exception e){
					Thread.currentThread().interrupt();
				}
			}
			//if its ready start processing
			else {
				this.startTime = System.currentTimeMillis();
				this.param.eval = this;				
				double correctCount=0; 
				svm.svm_cross_validation(this.problem, this.param, this.folds, this.target); 
				for (int i=0; i < this.problem.l; i++) { 
					if (this.problem.y[i] == target[i]) 
						correctCount++;
				}
				accuracy = correctCount / this.problem.l; 
				this.endTime = System.currentTimeMillis();
				this.ready = false;
			}
		}
		
	}
	
	public  void setProblem(svm_problem problem){
		this.problem = problem;
	}
	
	public  void setTarget(double[] target){
		this.target = target;
	}
	
	public  void setParameter(svm_parameter svmparam){
		this.param = svmparam;
	}
	
	public  void setSVM(svm svmO){
		this.svmObject = svmO;
	}
	
	public  void setData(KernelData data){
		this.data = data;
	}
	
	public  void setIndividual(GPIndividual individual){
		this.individual = individual;
	}
	
	public  void setThreadNumber(int thread){
		this.threadNumber = thread;
	}
	
	public void setNumberOfFolds(int folds){
		this.folds = folds;
	}
	
	public  synchronized void setReady(){
		this.accuracy =0;
		this.ready = true;
		notifyAll();
	}
	
	public synchronized boolean isReady(){
		return this.ready;
	}
	
	public long getStartTime(){
		return this.startTime;
	}
	
	public long getEndTime(){
		return this.endTime;
	}
	
	public boolean getReady(){
		return this.ready;
	}
	
	public double getAccuracy(){
		return this.accuracy;
	}
	
	

}
