package evolvekernel.func; 

import ec.*; 
import ec.gp.*; 
import ec.util.*;
import evolvekernel.*;
import libsvm.*;

/**
 * This class acts as Node operator to do Addition of two kernesl
 * Kernel1 + Kernel2
 * @author udaykamath
 *
 */
public class Add extends GPNode { 
	
	public String toString() { return "+"; } 
	
	public void checkConstraints(final EvolutionState state,
								 final int tree, 
								 final GPIndividual typicalIndividual,
								 final Parameter individualBase)
	{
		super.checkConstraints(state,tree,typicalIndividual,individualBase); 
		if (children.length!=2) 
			state.output.error("Incorrect number of children for node " + 
							   toStringForError() + " at " + 
							   individualBase); 
	}
	
	public void eval(final EvolutionState state, 
					 final int thread, 
					 final GPData input, 
					 final ADFStack stack, 
					 final GPIndividual individual, 
					 final Problem problem)
	{
		KernelData sd = (KernelData)input; 
		
		children[0].eval(state,thread,input,stack,individual,problem); 
		svm_node[] x1 = sd.x; 
		
		children[1].eval(state,thread,input,stack,individual,problem);
		svm_node[] x2 = sd.x; 
		
		sd.val = 0;

		int xLen = x1.length; 
		int yLen = x2.length; 
		int i=0, j=0; 
		while (i < xLen && j < yLen) { 
			if (x1[i].index == x2[j].index) { 
				sd.val += x1[i].value + x2[j].value; 
				i++; j++; 
			}
			else if (x1[i].index < x2[j].index) 
				i++; 
			else 
				j++;
			
		}
		
		/*
		if ((len%2)==0) { 
			for (int i=0; i < len; i+=2) { 
				sd.val += x1[i].value + x2[i].value;
				sd.val += x1[i+1].value + x2[i+1].value;
			}
		}
		else { 
			for (int i=0; i < len; i++) { 
				sd.val += x1[i].value + x2[i].value;
			}
		}
		 */
	}
}