package evolvekernel.func;

import libsvm.svm_node;
import ec.EvolutionState;
import ec.Problem;
import ec.gp.ADFStack;
import ec.gp.GPData;
import ec.gp.GPIndividual;
import ec.gp.GPNode;
import ec.util.Parameter;
import evolvekernel.Functions;
import evolvekernel.KernelData;
/**
 * The ANOVA kernel is also a radial basis function kernel,
 *  just as the Gaussian and Laplacian kernels. 
 * Kernel Dot Product   Sum (EXP(-sigma* (X^k-Y^K)^D
 * @author udaykamath
 *
 */
public class AnovaKernel extends GPNode{
public String toString() { return "ANOVA"; } 
	
	public void checkConstraints(final EvolutionState state,
								 final int tree, 
								 final GPIndividual typicalIndividual,
								 final Parameter individualBase)
	{
		super.checkConstraints(state,tree,typicalIndividual,individualBase); 
		if (children.length!=4) 
			state.output.error("Incorrect number of children for node " + 
							   toStringForError() + " at " + 
							   individualBase); 
	}
	
	public void eval(final EvolutionState state, 
					 final int thread, 
					 final GPData input, 
					 final ADFStack stack, 
					 final GPIndividual individual, 
					 final Problem problem)
	{
		KernelData sd = (KernelData)input; 
		
		children[0].eval(state,thread,input,stack,individual,problem); 
		svm_node[] x1 = sd.x; 
		
		children[1].eval(state,thread,input,stack,individual,problem); 
		svm_node[] x2 = sd.y; 
		
		children[2].eval(state,thread,input,stack,individual,problem); 
		double sigma = sd.val;
		
		children[3].eval(state,thread,input,stack,individual,problem); 
		int order = sd.order;
		
		double a = 0.0;
        double d = 0.0;
		 for (int k = 0; k < x1.length; k++)
         {
			 d = x1[k].value- x2[k].value;
             a += FastExp.exp(-sigma * d * d);
         }
         double result = FastPow.pow(a, order);     
         sd.val =  result;
	}
}
