package evolvekernel.func;

import libsvm.svm_node;
import ec.EvolutionState;
import ec.Problem;
import ec.gp.ADFStack;
import ec.gp.GPData;
import ec.gp.GPIndividual;
import ec.gp.GPNode;
import ec.util.Parameter;
import evolvekernel.Functions;
import evolvekernel.KernelData;
/**
* Cauchy for input X and Y is given by
* Kernel Dot Product =  1 + X*Y*(1+minv) - ((X+Y)/2)*minv^2 + (minv^3)/3
* @author udaykamath
*
*/
public class BSpline extends GPNode{
	public String toString() {
		return "BSpline";
	}

	public void checkConstraints(final EvolutionState state, final int tree,
			final GPIndividual typicalIndividual, final Parameter individualBase) {
		super.checkConstraints(state, tree, typicalIndividual, individualBase);
		if (children.length != 3)
			state.output.error("Incorrect number of children for node "
					+ toStringForError() + " at " + individualBase);
	}

	public void eval(final EvolutionState state, final int thread,
			final GPData input, final ADFStack stack,
			final GPIndividual individual, final Problem problem) {
		KernelData sd = (KernelData) input;

		children[0].eval(state, thread, input, stack, individual, problem);
		svm_node[] x = sd.x;

		children[1].eval(state, thread, input, stack, individual, problem);
		svm_node[] y = sd.y;

		children[2].eval(state, thread, input, stack, individual, problem);
		int order = sd.order;
		 double k = 1.0;
         int n = 2*order+1;

         for (int p = 0; p < x.length; p++)
             k *= Functions.bSpline(n, x[p].value - y[p].value);


		sd.val = k;
		
	}
}
