package evolvekernel.func;

import libsvm.svm_node;
import ec.EvolutionState;
import ec.Problem;
import ec.gp.ADFStack;
import ec.gp.GPData;
import ec.gp.GPIndividual;
import ec.gp.GPNode;
import ec.util.Parameter;
import evolvekernel.Functions;
import evolvekernel.KernelData;
/**
 * Bessel Kernel is a complex Kernel with kernel dotproduct given by
 * logic below
*  lim <- 1/(gamma(order+1)*2^(order))
*        bkt <- sigma*sqrt(-(2*crossprod(X,Y) - crossprod(X) - crossprod(Y)))
*        if(bkt < 10e-5)
*          res <- lim
*        else
*         res <- besselJ(bkt,order)*(bkt^(-order))
*       return((res/lim)^degree)
 * @author udaykamath
 *
 */
public class Bessel extends GPNode{
	public String toString() { return "Bessel"; } 
	
	public void checkConstraints(final EvolutionState state,
								 final int tree, 
								 final GPIndividual typicalIndividual,
								 final Parameter individualBase)
	{
		super.checkConstraints(state,tree,typicalIndividual,individualBase); 
		if (children.length!=4) 
			state.output.error("Incorrect number of children for node " + 
							   toStringForError() + " at " + 
							   individualBase); 
	}
	
	public void eval(final EvolutionState state, 
					 final int thread, 
					 final GPData input, 
					 final ADFStack stack, 
					 final GPIndividual individual, 
					 final Problem problem)
	{
		KernelData sd = (KernelData)input; 
		
		children[0].eval(state,thread,input,stack,individual,problem); 
		svm_node[] x1 = sd.x; 
		
		children[1].eval(state,thread,input,stack,individual,problem); 
		svm_node[] x2 = sd.y; 
		
		children[2].eval(state,thread,input,stack,individual,problem); 
		double sigma = sd.val;
		
		children[3].eval(state,thread,input,stack,individual,problem); 
		int order = sd.order;
		
		double result =0;
		double norm = 0;
		 for (int k = 0; k < x1.length; k++)
         {
             double d = x1[k].value - x2[k].value;
             norm += d * d;
         }
         norm = Math.sqrt(norm);

         result = Functions.besselJ(order, sigma * norm) /
         FastPow.pow(norm, -norm * order);     
         sd.val =  result;
	}
}