package evolvekernel.func;

import libsvm.svm_node;
import ec.EvolutionState;
import ec.Problem;
import ec.gp.ADFStack;
import ec.gp.GPData;
import ec.gp.GPIndividual;
import ec.gp.GPNode;
import ec.util.Parameter;
import evolvekernel.KernelData;
/**
 ChiSquare for input X and Y is given by
* Kernel Dot Product = 1 - Sum(Xi-Yi)^2/0.5*(Xi +Yi)
* @author udaykamath
*
*/
public class ChiSquare extends GPNode { 
	
	public String toString() { return "CHI"; } 
	
	public void checkConstraints(final EvolutionState state,
								 final int tree, 
								 final GPIndividual typicalIndividual,
								 final Parameter individualBase)
	{
		super.checkConstraints(state,tree,typicalIndividual,individualBase); 
		if (children.length!=2) 
			state.output.error("Incorrect number of children for node " + 
							   toStringForError() + " at " + 
							   individualBase); 
	}
	
	public void eval(final EvolutionState state, 
					 final int thread, 
					 final GPData input, 
					 final ADFStack stack, 
					 final GPIndividual individual, 
					 final Problem problem)
	{
		KernelData sd = (KernelData)input; 
		
		children[0].eval(state,thread,input,stack,individual,problem); 
		svm_node[] x1 = sd.x; 
		
		children[1].eval(state,thread,input,stack,individual,problem); 
		svm_node[] x2 = sd.y; 
		
		double sum = 0.0;
		if(x1.length != x2.length)
			System.out.println("Arrays are not of same length!");
        for (int i = 0; i < x1.length; i++)
        {
            double num = x1[i].value - x2[i].value;
            sum += (num * num) / (0.5 * (x1[i].value + x2[i].value));
        }

        sd.val =  1.0 - sum;
	}
}