package evolvekernel.func;

import libsvm.svm_node;
import ec.EvolutionState;
import ec.Problem;
import ec.gp.ADFStack;
import ec.gp.GPData;
import ec.gp.GPIndividual;
import ec.gp.GPNode;
import ec.util.Parameter;
import evolvekernel.KernelData;
/**
 * FourierKernel for input X and Y is given by
 * Kernel Dot Product =(1 - sigma^2)/2*(1 - 2*sigma*cos(x - y) + sigma^2)
 * @author udaykamath
 *
 */
public class FourierKernel extends GPNode{
	public static final double OLD_MIN=-1;
	public static final double OLD_MAX=1;
	public static final double NEW_MIN=0;
	public static final double NEW_MAX=2*Math.PI;
	public String toString() {
		return "Fourier";
	}

	public void checkConstraints(final EvolutionState state, final int tree,
			final GPIndividual typicalIndividual, final Parameter individualBase) {
		super.checkConstraints(state, tree, typicalIndividual, individualBase);
		if (children.length != 3)
			state.output.error("Incorrect number of children for node "
					+ toStringForError() + " at " + individualBase);
	}

	
	public void eval(final EvolutionState state, final int thread,
			final GPData input, final ADFStack stack,
			final GPIndividual individual, final Problem problem) {
		KernelData sd = (KernelData) input;

		children[0].eval(state, thread, input, stack, individual, problem);
		svm_node[] x = sd.x;

		children[1].eval(state, thread, input, stack, individual, problem);
		svm_node[] y = sd.y;

		children[2].eval(state, thread, input, stack, individual, problem);
		double gamma = sd.val;

		double result = 0.0;
		double gamma_squared=gamma*gamma;
		for (int i = 0; i < x.length; i++) {
			double newX = scale(x[i].value);
			double newY = scale(y[i].value);
			result*=(1.0-gamma_squared)
		      / (2.0-(4.0*gamma*Math.cos(newX - newY))+2.0*gamma_squared);
		}
		sd.val = result;
	}
	
	/**
	 * We convert -1,1 to 0,2PI range for Fourierkernel
	 * @param oldValue
	 * @return
	 */private double scale(double oldValue){
		double newValue = 0;
		newValue = ( (oldValue - OLD_MIN) / (OLD_MAX - OLD_MIN) ) * (NEW_MAX - NEW_MIN);
		return newValue;
	}
	
	

}
