package evolvekernel.func; 

import ec.*; 
import ec.gp.*; 
import ec.util.*;
import evolvekernel.*;
import libsvm.*;
/**
 * This is the gaussian form standard
 * Kernel Dot Product = EXP(-sigma * normalization(X-Y)^2)
 * @author udaykamath
 *
 */
public class Gaussian extends GPNode { 
	public String toString() { return "G"; } 
	
	public void checkConstraints(final EvolutionState state,
								 final int tree, 
								 final GPIndividual typicalIndividual,
								 final Parameter individualBase)
	{
		super.checkConstraints(state,tree,typicalIndividual,individualBase); 
		if (children.length!=3) 
			state.output.error("Incorrect number of children for node " + 
							   toStringForError() + " at " + 
							   individualBase); 
	}
	
	public double getSum(svm_node[] x, svm_node[] y)
	{
		
	 	double sum = 0;
		int xlen = x.length;
		int ylen = y.length;
		int i = 0;
		int j = 0;
		
		while (i < xlen && j < ylen)
		{
			if(x[i].index == y[j].index)
			{
				double d = x[i++].value - y[j++].value;
				sum += d*d;
			}
			else if(x[i].index > y[j].index)
			{
				sum += y[j].value * y[j].value;
				++j;
			}
			else
			{
				sum += x[i].value * x[i].value;
				++i;
			}
		}
		
		while  (i < xlen)
		{
			sum += x[i].value * x[i].value;
			++i;
		}
		
		while (j < ylen)
		{
			sum += y[j].value * y[j].value;
			++j;
		}
		
		return sum; 
	}
	
	
	public void eval(final EvolutionState state, 
					 final int thread, 
					 final GPData input, 
					 final ADFStack stack, 
					 final GPIndividual individual, 
					 final Problem problem)
	{
		KernelData sd = (KernelData)input; 
		
		children[0].eval(state,thread,input,stack,individual,problem); 
		svm_node[] x = sd.x;
		
		children[1].eval(state,thread,input,stack,individual,problem); 
		svm_node[] y = sd.y;
		
		children[2].eval(state,thread,input,stack,individual,problem); 
		double sigma = sd.val;
			
		double sum = getSum(x,y); 
	
		
		//sd.val = FastExp.exp(-sigma * sum); 
		sd.val = FastExp.exp(-sigma * sum); 
	}
}

