package evolvekernel.func;

import libsvm.svm_node;
import ec.EvolutionState;
import ec.Problem;
import ec.gp.ADFStack;
import ec.gp.GPData;
import ec.gp.GPIndividual;
import ec.gp.GPNode;
import ec.util.Parameter;
import evolvekernel.KernelData;
/**
 * HistogramIntersection Kernel for input X and Y is given by
 * Kernel Dot Product = Sum(Min(X,Y))
 * @author udaykamath
 *
 */
public class HistogramIntersection extends GPNode {
	public String toString() {
		return "HiInt";
	}

	public void checkConstraints(final EvolutionState state, final int tree,
			final GPIndividual typicalIndividual, final Parameter individualBase) {
		super.checkConstraints(state, tree, typicalIndividual, individualBase);
		if (children.length != 2)
			state.output.error("Incorrect number of children for node "
					+ toStringForError() + " at " + individualBase);
	}

	public void eval(final EvolutionState state, final int thread,
			final GPData input, final ADFStack stack,
			final GPIndividual individual, final Problem problem) {
		KernelData sd = (KernelData) input;

		children[0].eval(state, thread, input, stack, individual, problem);
		svm_node[] x = sd.x;

		children[1].eval(state, thread, input, stack, individual, problem);
		svm_node[] y = sd.y;

		

		double sum = 0.0;

		for (int i = 0; i < x.length; i++) {
//			sum += Math.min(FastPow.pow(Math.abs(x[i].value), alpha), FastPow
//					.pow(Math.abs(y[i].value), beta));
			sum += Math.min(x[i].value, y[i].value);
		}
		sd.val = sum;
		
	}
}
