package evolvekernel.func;


import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;

import ec.EvolutionState;
import ec.Problem;
import ec.gp.ADFStack;
import ec.gp.ERC;
import ec.gp.GPData;
import ec.gp.GPIndividual;
import ec.gp.GPNode;
import ec.util.Code;
import ec.util.DecodeReturn;
import ec.util.Parameter;
import evolvekernel.KernelData;
/**
 * This is the ERC class for various integer based numbers
 * @author udaykamath
 *
 */
public class IntegerERC extends ERC{
	int x;
	public void resetNode(final EvolutionState state, final int thread) {
		x = state.random[thread].nextInt(7);
	}
	
	public String name() {
		return "'" + new Integer(x).toString() + "'";
	}
	public String toString() {
		return "" + new Integer(x).toString() + "";
	}

	public int nodeHashCode() {
		return this.getClass().hashCode() + Integer.bitCount(x);
	}

	public boolean nodeEquals(final GPNode node) {
		if (this.getClass() != node.getClass())
			return false;
		IntegerERC node1 = (IntegerERC) node;
		if (node1.x == this.x)
			return true;
		return false;
	}

	public void readNode(final EvolutionState state, final DataInput dataInput)
			throws IOException {
		x = dataInput.readInt();
	}

	public void writeNode(final EvolutionState state,
			final DataOutput dataOutput) throws IOException {
		dataOutput.writeInt(x);
	}

	public String encode() {
		return Code.encode(x);
	}

	public boolean decode(DecodeReturn dret) {
		// store the position and the string in case they
		// get modified by Code.java
		int pos = dret.pos;
		String data = dret.data;

		// decode
		Code.decode(dret);

		if (dret.type != DecodeReturn.T_INT) // uh oh!
		{
			// restore the position and the string; it was an error
			dret.data = data;
			dret.pos = pos;
			return false;
		}

		// store the data
		x = new Integer(data).intValue();

		return true;
	}


    public void checkConstraints(final EvolutionState state,
        final int tree,
        final GPIndividual typicalIndividual,
        final Parameter individualBase)
        {
        super.checkConstraints(state,tree,typicalIndividual,individualBase);
        if (children.length!=0)
            state.output.error("Incorrect number of children for node " + 
                toStringForError() + " at " +
                individualBase);
        }

    public void eval(final EvolutionState state,
        final int thread,
        final GPData input,
        final ADFStack stack,
        final GPIndividual individual,
        final Problem problem)
        {
    	KernelData data = ((KernelData) (input));
		
		data.order = x;
		
        }
    
    public int getRandom(final EvolutionState state, final int thread){
		return state.random[thread].nextInt(150) ;
	}
    
}

