package evolvekernel.func;

import libsvm.svm_node;
import ec.EvolutionState;
import ec.Problem;
import ec.gp.ADFStack;
import ec.gp.GPData;
import ec.gp.GPIndividual;
import ec.gp.GPNode;
import ec.util.Parameter;
import evolvekernel.KernelData;
/**
 * Laplacian Kernel for input X and Y is given by
 * Kernel Dot Product = EXP(-sigma * normalization(X-Y))
 * @author udaykamath
 *
 */
public class Laplacian extends GPNode { 
	public String toString() { return "Laplace"; } 
	
	public void checkConstraints(final EvolutionState state,
								 final int tree, 
								 final GPIndividual typicalIndividual,
								 final Parameter individualBase)
	{
		super.checkConstraints(state,tree,typicalIndividual,individualBase); 
		if (children.length!=3) 
			state.output.error("Incorrect number of children for node " + 
							   toStringForError() + " at " + 
							   individualBase); 
	}
	
	public double getSum(svm_node[] x, svm_node[] y)
	{
		
	 	double sum = 0;
		int xlen = x.length;
		int i = 0;		
		while(i < xlen){
				double d = x[i].value - y[i].value;
				sum += d*d;		
				i = i+1;
		}
		
		
		return sum; 
	}
	
	
	public void eval(final EvolutionState state, 
					 final int thread, 
					 final GPData input, 
					 final ADFStack stack, 
					 final GPIndividual individual, 
					 final Problem problem)
	{
		KernelData sd = (KernelData)input; 
		
		children[0].eval(state,thread,input,stack,individual,problem); 
		svm_node[] x = sd.x;
		
		children[1].eval(state,thread,input,stack,individual,problem); 
		svm_node[] y = sd.y;
		
		children[2].eval(state,thread,input,stack,individual,problem); 
		double sigma = sd.val;
			
		double sum = getSum(x,y); 
	
		
		//sd.val = FastExp.exp(-sigma * sum); 
		sd.val = FastExp.exp(-sigma * Math.sqrt(sum)); 
	}
}
