package evolvekernel.func;

import libsvm.svm_node;
import ec.EvolutionState;
import ec.Problem;
import ec.gp.ADFStack;
import ec.gp.GPData;
import ec.gp.GPIndividual;
import ec.gp.GPNode;
import ec.util.Parameter;
import evolvekernel.Functions;
import evolvekernel.KernelData;
/**
 * MultiQuadratic Kernel for input X and Y is given by
 * Kernel Dot Product = SquareRoot(Normalization(X-Y)^2 + constant*constant)
 * Simplified as Exp -1/EXP +1 of dot product
 * @author udaykamath
 *
 */
public class MultiQuadratic extends GPNode{
	public String toString() {
		return "MQ";
	}

	public void checkConstraints(final EvolutionState state, final int tree,
			final GPIndividual typicalIndividual, final Parameter individualBase) {
		super.checkConstraints(state, tree, typicalIndividual, individualBase);
		if (children.length != 3)
			state.output.error("Incorrect number of children for node "
					+ toStringForError() + " at " + individualBase);
	}

	
	public void eval(final EvolutionState state, final int thread,
			final GPData input, final ADFStack stack,
			final GPIndividual individual, final Problem problem) {
		KernelData sd = (KernelData) input;

		children[0].eval(state, thread, input, stack, individual, problem);
		svm_node[] x = sd.x;

		children[1].eval(state, thread, input, stack, individual, problem);
		svm_node[] y = sd.y;

		children[2].eval(state, thread, input, stack, individual, problem);
		double theta = sd.val;

		double norm = 0.0;
		double result = 0.0;
		for (int i = 0; i < x.length; i++) {
			double d = x[i].value - y[i].value;
			norm += d * d;
		}

		result =Math.sqrt(norm + theta*theta);
		
		sd.val = result;

	}
}
