package evolvekernel.func; 

import ec.*; 
import ec.gp.*; 
import ec.util.*;
import evolvekernel.*;
import libsvm.*;
/**
 * This is a kernel operator which takes input a kernel and
 * converts into polynomial of given degree.
 * @author udaykamath
 *
 */
public class Poly extends GPNode { 
	
	public String toString() { return "P"; } 
	
	public void checkConstraints(final EvolutionState state,
								 final int tree, 
								 final GPIndividual typicalIndividual,
								 final Parameter individualBase)
	{
		super.checkConstraints(state,tree,typicalIndividual,individualBase); 
		if (children.length!=3) 
			state.output.error("Incorrect number of children for node " + 
							   toStringForError() + " at " + 
							   individualBase); 
	}
	
	public void eval(final EvolutionState state, 
					 final int thread, 
					 final GPData input, 
					 final ADFStack stack, 
					 final GPIndividual individual, 
					 final Problem problem)
	{
		KernelData sd = (KernelData)input; 
		
		children[0].eval(state,thread,input,stack,individual,problem); 
		svm_node[] x1 = sd.x; 
		
		children[1].eval(state,thread,input,stack,individual,problem); 
		svm_node[] x2 = sd.y; 
			
		children[2].eval(state,thread,input,stack,individual,problem); 
		double power = sd.val;
		
		
		double gamma = 0.005;
		
		//System.out.println(Functions.dotProduct(x1,x2) + "\t" + power); 
		double x[] = new double[x1.length];
		for(int i=0; i< x1.length; i++)
			x[i] = x1[i].value;
		
		double y[] = new double[x2.length];
		for(int i=0; i< x2.length; i++)
			y[i] = x2[i].value;
		
		//sd.val = Math.pow(Functions.dotProduct(x1, x2), power); 
		double dotvalue = Functions.dotProduct(x1, x2);
		sd.val = FastPow.pow(dotvalue, power); 
	}
}
	
