package evolvekernel.func;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.File;
import java.io.IOException;

import ec.EvolutionState;
import ec.Evolve;
import ec.Problem;
import ec.gp.ADFStack;
import ec.gp.ERC;
import ec.gp.GPData;
import ec.gp.GPIndividual;
import ec.gp.GPInitializer;
import ec.gp.GPNode;
import ec.util.Code;
import ec.util.DecodeReturn;
import ec.util.Parameter;
import ec.util.ParameterDatabase;
import evolvekernel.KernelData;
/**
 * This class is another way of using ERC for C, gamma etc. This can specify
 * Range [-5,15] and base {2} or {10} etc. This searches in the exact
 * variance in random manner, i.e. between max and min does sampling of 
 * values. This could be extended like the RangeBasedIncrement for having
 * "maxCount" in future.
 * @author udaykamath
 *
 */
public class RangeBased extends ERC{
	
	/* can be -5 */
	protected int minRange;
	/* can be +10*/
	protected int maxRange;
	/* base of the power , can be 10,2 etc*/
	protected int base;
	
	public double x;
	
	

	public void setup(final EvolutionState state, final Parameter base)
    {
	    super.setup(state, base);
	
	    RangeBasedConstraints constraintsObj = (RangeBasedConstraints)constraints(((GPInitializer)state.initializer));
	    
	    //get the values
	    this.maxRange = constraintsObj.maxRange;
	    this.minRange = constraintsObj.minRange;
	    this.base = constraintsObj.powerBase;
	    
    }

	public void resetNode(final EvolutionState state, final int thread) {		
		int n = maxRange - minRange + 1;
		int i = state.random[thread].nextInt() % n;
		int nextRandomInRange =  minRange + i;
		//make it power of 2
		x = Math.pow(base, nextRandomInRange);
		System.out.println("Value: " + x);
	}

	public int nodeHashCode() {
		return this.getClass().hashCode() + Float.floatToIntBits((float) x);
	}

	public boolean nodeEquals(final GPNode node) {
		if (this.getClass() != node.getClass())
			return false;
		RangeBased node1 = (RangeBased) node;
		if (node1.x == this.x)
			return true;
		return false;
	}

	public void readNode(final EvolutionState state, final DataInput dataInput)
			throws IOException {
		x = dataInput.readDouble();
	}

	public void writeNode(final EvolutionState state,
			final DataOutput dataOutput) throws IOException {
		dataOutput.writeDouble(x);
	}

	public String encode() {
		return Code.encode(x);
	}

	public boolean decode(DecodeReturn dret) {
		// store the position and the string in case they
		// get modified by Code.java
		int pos = dret.pos;
		String data = dret.data;

		// decode
		Code.decode(dret);

		if (dret.type != DecodeReturn.T_DOUBLE) // uh oh!
		{
			// restore the position and the string; it was an error
			dret.data = data;
			dret.pos = pos;
			return false;
		}

		// store the data
		x = dret.d;

		return true;
	}

	public String name() {
		return "";
	} // I'm the only ERC class, this is fine

	public String toStringForHumans() {
		return "(" + (float) x + ")";
	}

	public void eval(final EvolutionState state, final int thread,
			final GPData input, final ADFStack stack,
			final GPIndividual individual, final Problem problem) {
		KernelData kd = (KernelData) input;
		kd.val = x;
	}
	
	public static void main(String[] args){
		try {
			ParameterDatabase parameters; 
			parameters = new ParameterDatabase(new File(new File("Testsvm.kernel").getAbsolutePath())); 
			EvolutionState state = Evolve.initialize(parameters,1); 
			state.setup(null,null); 
			RangeBased range = new RangeBased();
			//range.setup(evoState, new Parameter(""));
			range.maxRange =5;
			range.minRange=-5;
			range.base =2;
			for(int i=0; i< 500; i++)
				range.resetNode(state, 1);
		}
		catch(Exception e){
			e.printStackTrace();
		}
	}

}
