package evolvekernel.func;

import ec.EvolutionState;
import ec.gp.GPInitializer;
import ec.gp.GPNode;
import ec.gp.GPNodeConstraints;
import ec.gp.GPType;
import ec.util.Parameter;

/**
 * This code extends the GPNodeConstraints constraints to get
 * Power Base: Whether its 2,10 etc
 * Max Range: Limit on where maximum it can go in the power base, 
 * i.e. powerbase ^ maxRange is maximum it can go
 * Min Range: Limit on where minimum it can go in the power base, 
 * i.e. powerbase ^ minRange is minimum it can go
 * 
 * @author udaykamath
 * 
 */

public class RangeBasedConstraints extends GPNodeConstraints{
	
	
    public static String P_POWERBASE ="base";
	public static String P_MAXRANGE ="max";
	public static String P_MINRANGE="min";
	
	protected int minRange;
	/* can be +10*/
	protected int maxRange;
	/* base of the power , can be 10,2 etc*/
	protected int powerBase;
	
    

    /** This must be called <i>after</i> the GPTypes have been set up. */
    public  void setup(final EvolutionState state, final Parameter base)
        {
    	super.setup(state,base);
        // determine my constraints -- at this point, the constraints should have been loaded.
        String base1 = state.parameters.getString(base.push(P_POWERBASE),
            null);
        powerBase = new Integer(base1).intValue();
        
        //get the maxRange
        String max = state.parameters.getString(base.push(P_MAXRANGE),
                null);
        maxRange = new Integer(max).intValue();
        
        //get the minRange
        String min = state.parameters.getString(base.push(P_MINRANGE),
                null);
        
        // ...because I promised when I called typeFor(...)
        state.output.exitIfErrors();    
        }


    /** You must guarantee that after calling constraintsFor(...) one or
        several times, you call state.output.exitIfErrors() once. */

    public static GPNodeConstraints constraintsFor(final String constraintsName,
        final EvolutionState state)
        {
        GPNodeConstraints myConstraints = (GPNodeConstraints)(((GPInitializer)state.initializer).nodeConstraintRepository.get(constraintsName));
        if (myConstraints==null)
            state.output.error("The GP node constraint \"" + constraintsName + "\" could not be found.");
        return myConstraints;
        }
}
