package evolvekernel.func;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import ec.EvolutionState;
import ec.Evolve;
import ec.gp.GPInitializer;
import ec.util.Parameter;
import ec.util.ParameterDatabase;

/**
 * This class can be used to set incremental range based search.
 * [-5,15] can be the range, increment can be 1, powerbase can be 2,10 etc
 * Can also specify the number of times it can stay in the range for example
 * if it gets stuck in -4 to -3 i.e. 1/2^4 to 1/2^3, for N (user defined)
 * it automatically gets reset to jump randomly in the large sequence.
 * This adds to stochastic nature.
 * 
 * @author udaykamath
 *
 */public class RangeBasedStepIncremental extends RangeBased{
	/** The natural logarithm of 2. */
	public static double log2 = Math.log(2);
	
	private int stepSize;
	
	private List sortedSequenceList;
	private Map<Integer, Integer> currentCount= new HashMap<Integer,Integer>();
	private int maxCount;
	
	protected List getSortedList(){
		if(this.sortedSequenceList == null){
			this.sortedSequenceList = new ArrayList();
			for(int i=minRange; i<= maxRange; i=i+ stepSize){
				this.sortedSequenceList.add( new Integer(i));
			}
		}
		return this.sortedSequenceList;
	}
	
	public void setup(final EvolutionState state, final Parameter base)
    {
	    super.setup(state, base);
	    //get the step size
	    RangeBasedConstraintsIncremental constraintsObj = (RangeBasedConstraintsIncremental)constraints(((GPInitializer)state.initializer));
	    this.stepSize = constraintsObj.stepSize;
	    this.maxCount = constraintsObj.maxCount;
	    	    
	    
    }
	
	

	public void resetNode(final EvolutionState state, final int thread) {		
		//calculate the current value in normal
		int currentVal= 0;
		
		if(this.base == 10){
			currentVal = (int)Math.log10(x);
			
		}
		else {
			currentVal = (int)log2(x);
		}
		if(currentVal > this.maxRange)
			currentVal = maxRange;
		if(currentVal < this.minRange)
			currentVal = minRange;
		//increment the count 
		Integer count = this.currentCount.get(currentVal);
		if(count != null)
			count = count +1;
		else
			count =0;
		//check the max
		if(count < maxCount){
			this.currentCount.put(currentVal, count);
		}
		//if max is reached clear it and find random point from range
		else{
			this.currentCount.put(currentVal, 0);
			//find a random new jump
			int index = state.random[thread].nextInt(this.getSortedList().size());
			currentVal = (Integer)this.getSortedList().get(index);
		}
		
		boolean found = false;
		int numberOfElements = this.getSortedList().size();
		//see where you are in sorted sequence of ranges
		for(int i =0; i< numberOfElements; i++){
			//get the element
			Integer value = (Integer)this.getSortedList().get(i);
			//check if what value of currentVal i am
			if(currentVal < value ){
				int low =0,high =0;
				//if i am inside the range list
				if(i >= 1 && i< numberOfElements -1) {
				 low = (Integer)this.getSortedList().get(i-1);
				 high = (Integer)this.getSortedList().get(i+1);
				}
				//if i am on the left border
				if(i ==0  ) {
					 low = (Integer)this.getSortedList().get(0);
					 high = (Integer)this.getSortedList().get(i+ 1);
				}
				//if i am on the right border
				if(i == this.getSortedList().size()-1) {
					 low = (Integer)this.getSortedList().get(i-1);
					 high = (Integer)this.getSortedList().get(i);
				}
				//check for high low 
				int temp = high;
				if(temp < low){
					high = low;
					low = temp;
				}
				x = this.getRandom(state, thread, high, low);
				found = true;
				System.out.println("low:" + low + " high:" + high + " currentRandom: " +  " reset: " + x);
				break;
			}
			
		}
		//if not found take the last one 
		if(!found){
			int low = (Integer)this.getSortedList().get(numberOfElements-2);
			int high = (Integer)this.getSortedList().get(numberOfElements-1);
			//make it power of 2
			x = this.getRandom(state, thread, high, low);
		}
	}
	
	
	public double getRandom(final EvolutionState state, final int thread, int high, int low){
		//find a number uniformly in this range
		double n = high - low + 1;
		double rand = state.random[thread].nextDouble() % n;
		double nextRandomInRange =  low + rand;
		return Math.pow(base, nextRandomInRange);
	}
	/**
	 * Returns the logarithm of a for base 2.
	 * 
	 * @param a
	 *            a double
	 */
	public /* @pure@ */double log2(double a) {

		return Math.log(a) / log2;
	}
	
	public static void main(String[] args){
		try {
			ParameterDatabase parameters; 
			parameters = new ParameterDatabase(new File(new File("Testsvm.kernel").getAbsolutePath())); 
			EvolutionState state = Evolve.initialize(parameters,1); 
			state.setup(null,null); 
			RangeBasedStepIncremental range = new RangeBasedStepIncremental();
			//range.setup(evoState, new Parameter(""));
			range.maxRange =5;
			range.minRange=-5;
			range.base =2;
			range.stepSize =1;
			range.maxCount =5;
			for(int i=0; i< 5000; i++){
				range.resetNode(state, 1);
				//System.out.println("Current index:" + i + "range: " + range.x);
				
			}
		}
		catch(Exception e){
			e.printStackTrace();
		}
	}

}
