package evolvekernel.func; 
import ec.*;
import ec.gp.*;
import ec.util.*;
import java.io.*;
import evolvekernel.*;
/**
 * One version of ERC (random double/real) number that can be used
 * for parameters of C, gamma, order etc in kernel evaluation.
 * @author udaykamath
 *
 */
public class RegERC extends ERC
    {
		public double x;
		
		public void resetNode(final EvolutionState state, final int thread)
        { 
			x = state.random[thread].nextDouble() * 50;
		}

    public int nodeHashCode()
        {
        return this.getClass().hashCode() + Float.floatToIntBits((float)x);
        }

    public boolean nodeEquals(final GPNode node)
        {
        if (this.getClass() != node.getClass()) return false;
		RegERC node1 = (RegERC)node; 
		if (node1.x == this.x)
			return true; 
		return false; 
		}

    public void readNode(final EvolutionState state, final DataInput dataInput) throws IOException
        {
        x = dataInput.readDouble();
        }

    public void writeNode(final EvolutionState state, final DataOutput dataOutput) throws IOException
        {
        dataOutput.writeDouble(x);
        }

    public String encode()
			{ return Code.encode(x); }

    public boolean decode(DecodeReturn dret)
        {
        // store the position and the string in case they
        // get modified by Code.java
        int pos = dret.pos;
        String data = dret.data;

        // decode
        Code.decode(dret);

        if (dret.type != DecodeReturn.T_DOUBLE) // uh oh!
            {
            // restore the position and the string; it was an error
            dret.data = data;
            dret.pos = pos;
            return false;
            }

        // store the data
		x = dret.d; 
		
		return true;
        }

    public String name() { return ""; } // I'm the only ERC class, this is fine

    public String toStringForHumans()
        { return "(" + (float)x + ")"; }

    public void eval(final EvolutionState state,
                     final int thread,
                     final GPData input,
                     final ADFStack stack,
                     final GPIndividual individual,
                     final Problem problem)
        {
		KernelData kd = (KernelData)input; 
		kd.val = x; 
        }
    }



