package evolvekernel.func; 

import ec.*; 
import ec.gp.*; 
import ec.util.*;
import evolvekernel.*;
import libsvm.*;
/**
 * Sigmoid Kernel for input X and Y is given by
 * Kernel Dot Product = tanh(alpha*X^T*Y + constant)
 * Simplified as Exp -1/EXP +1 of dot product
 * @author udaykamath
 *
 */
public class Sigmoid extends GPNode { 
	
	public String toString() { return "S"; } 
	
	public void checkConstraints(final EvolutionState state,
								 final int tree, 
								 final GPIndividual typicalIndividual,
								 final Parameter individualBase)
	{
		super.checkConstraints(state,tree,typicalIndividual,individualBase); 
		if (children.length!=3) 
			state.output.error("Incorrect number of children for node " + 
							   toStringForError() + " at " + 
							   individualBase); 
	}
	
	public void eval(final EvolutionState state, 
					 final int thread, 
					 final GPData input, 
					 final ADFStack stack, 
					 final GPIndividual individual, 
					 final Problem problem)
	{
		KernelData sd = (KernelData)input; 
		
		children[0].eval(state,thread,input,stack,individual,problem); 
		svm_node[] x1 = sd.x; 
		
		children[1].eval(state,thread,input,stack,individual,problem); 
		svm_node[] x2 = sd.y; 
		
		children[2].eval(state,thread,input,stack,individual,problem); 
		double kappa = sd.val;
		
//		children[3].eval(state,thread,input,stack,individual,problem); 
//		double theta = sd.val;
				
		double z = kappa * Functions.dotProduct(x1, x2) ;
//		+ theta;
		double e = FastExp.exp(2*z); 
		//double e = Math.exp(2*z); 
		sd.val = (e-1) / (e+1); 
	}
}
