package evolvekernel.func;

import libsvm.svm_node;
import ec.EvolutionState;
import ec.Problem;
import ec.gp.ADFStack;
import ec.gp.GPData;
import ec.gp.GPIndividual;
import ec.gp.GPNode;
import ec.util.Parameter;
import evolvekernel.KernelData;
/**
 * Spline Kernel for input X and Y is given by
 * Kernel Dot Product =1 + X*Y + X*Y min(X,Y) - (X+Y)/2 min(X,Y)^2 + 1/3*min(X,Y)^3
 * @author udaykamath
 *
 */
public class Spline extends GPNode {

	public String toString() {
		return "SPLN";
	}

	public void checkConstraints(final EvolutionState state, final int tree,
			final GPIndividual typicalIndividual, final Parameter individualBase) {
		super.checkConstraints(state, tree, typicalIndividual, individualBase);
		if (children.length != 2)
			state.output.error("Incorrect number of children for node "
					+ toStringForError() + " at " + individualBase);
	}

	public void eval(final EvolutionState state, final int thread,
			final GPData input, final ADFStack stack,
			final GPIndividual individual, final Problem problem) {
		KernelData sd = (KernelData) input;

		children[0].eval(state, thread, input, stack, individual, problem);
		svm_node[] x = sd.x;

		children[1].eval(state, thread, input, stack, individual, problem);
		svm_node[] y = sd.y;

		double k = 1;
		for (int i = 0; i < x.length; i++) {
			double min = Math.min(x[i].value, y[i].value);
			double xy = x[i].value * y[i].value;

			// prod{1}^d 1 + xy + xy*min - (x+y)/2 min + min/3}
			k *= 1.0 + xy + xy * min - ((x[i].value + y[i].value) / 2.0) * min
					* min + (min * min * min) / 3.0;
		}

		sd.val = k;
	}

}
