package evolvekernel.func;

import libsvm.svm_node;
import ec.EvolutionState;
import ec.Problem;
import ec.gp.ADFStack;
import ec.gp.GPData;
import ec.gp.GPIndividual;
import ec.gp.GPNode;
import ec.util.Parameter;
import evolvekernel.KernelData;

/**
 * StudentTKernel for input X and Y is given by
 * Kernel Dot Product = 1/1 + normal(X-Y)^d
 * d is the input order.
 * @author udaykamath
 *
 */

public class StudentTKernel extends GPNode {
	public String toString() {
		return "StudentT";
	}

	public void checkConstraints(final EvolutionState state, final int tree,
			final GPIndividual typicalIndividual, final Parameter individualBase) {
		super.checkConstraints(state, tree, typicalIndividual, individualBase);
		if (children.length != 3)
			state.output.error("Incorrect number of children for node "
					+ toStringForError() + " at " + individualBase);
	}

	public void eval(final EvolutionState state, final int thread,
			final GPData input, final ADFStack stack,
			final GPIndividual individual, final Problem problem) {
		KernelData sd = (KernelData) input;

		children[0].eval(state, thread, input, stack, individual, problem);
		svm_node[] x1 = sd.x;

		children[1].eval(state, thread, input, stack, individual, problem);
		svm_node[] x2 = sd.y;

		children[2].eval(state, thread, input, stack, individual, problem);
		int power = sd.order;

		double norm = 0.0;
		for (int i = 0; i < x1.length; i++) {
			double d = x1[i].value - x2[i].value;
			norm += d * d;
		}
		norm = Math.sqrt(norm);

		sd.val = 1.0 / (1.0 + FastPow.pow(norm, power));
	}

}
