package evolvekernel.func;

import ec.EvolutionState;
import ec.gp.GPInitializer;
import ec.gp.GPNode;
import ec.gp.GPNodeConstraints;
import ec.gp.GPType;
import ec.util.Parameter;

/**
 * This code extends the Rangebased constraints to get max count and step size
 * as two additional parameters.Step size is used to check on range increment
 * For range -5, to 15 in power base of 2, step size of 1 means sequence will
 * have {-5,-4,-3,-2,-1,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,14} as valid
 * permutation sequences. 
 * Max Count: gives more of exploitation in a neighbourhood. If it stays in neighbourhood
 * of [-4,-3] i.e. 2^-4 and 2^-3, we can stay there for as many values given in max count
 * max count can be 50-100. This helps exploitational sampling in particular neighbourhood for 
 * better values.
 * 
 * @author udaykamath
 * 
 */
public class RangeBasedConstraintsIncremental extends RangeBasedConstraints {

	public final static String P_MAXCOUNT = "max-count";
	public final static String P_STEPSIZE = "step-size";

	protected int stepSize;
	protected int maxCount;

	public String toString() {
		return name;
	}

	/**
	 * A little memory optimization: if GPNodes have no children, they are
	 * welcome to use share this zero-sized array as their children array.
	 */
	public GPNode zeroChildren[] = new GPNode[0];

	/** This must be called <i>after</i> the GPTypes have been set up. */
	public void setup(final EvolutionState state, final Parameter base) {
		super.setup(state, base);
		// determine my constraints -- at this point, the constraints should
		// have been loaded.
		// get the maxCount
		String maxCountString = state.parameters.getString(base
				.push(P_MAXCOUNT), null);
		maxCount = new Integer(maxCountString).intValue();

		// get the step size
		String step = state.parameters.getString(base.push(P_STEPSIZE), null);
		stepSize = new Integer(step).intValue();
	}

	/**
	 * You must guarantee that after calling constraintsFor(...) one or several
	 * times, you call state.output.exitIfErrors() once.
	 */

	public static GPNodeConstraints constraintsFor(
			final String constraintsName, final EvolutionState state) {
		GPNodeConstraints myConstraints = (GPNodeConstraints) (((GPInitializer) state.initializer).nodeConstraintRepository
				.get(constraintsName));
		if (myConstraints == null)
			state.output.error("The GP node constraint \"" + constraintsName
					+ "\" could not be found.");
		return myConstraints;
	}
}
